<?php

namespace Drupal\search_replace_aliases\Form;

use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\path_alias\Entity\PathAlias;

/**
 * Implements the Search Replace Aliases form.
 */
class SearchReplaceAliasesForm extends FormBase {

  /**
   * Defines the form ID.
   */
  public function getFormId() {
    return 'search_replace_aliases_form';
  }

  /**
   * Builds the form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // Mensaje de advertencia sobre la redirección.
    $form['redirect_warning'] = [
      '#markup' => $this->t('<strong>¡Atención!</strong> Si deseas crear una redirección, asegúrate de que el módulo <em>Redirect</em> esté correctamente instalado y configurado.'),
      '#prefix' => '<div class="warning-message">',
      '#suffix' => '</div>',
    ];

    // Campos del formulario para cadenas de búsqueda y reemplazo.
    $form['search_string'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Buscar Alias'),
      '#description' => $this->t('Ingresa el fragmento del alias que deseas buscar.'),
      '#required' => TRUE,
    ];

    $form['replace_string'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Reemplazar Con'),
      '#description' => $this->t('Ingresa la cadena con la que deseas reemplazar.'),
      '#required' => TRUE,
    ];

    // Campo oculto para rastrear el paso de vista previa.
    $form['preview_step'] = [
      '#type' => 'hidden',
      '#value' => $form_state->get('preview_step') ? $form_state->get('preview_step') : '0',
    ];

    // Ajustar el botón de envío según el paso de vista previa.
    if ($form_state->get('preview_step') == '1') {
      // Solo mostrar el botón después de la lista de vista previa.
      $aliases_to_change = $form_state->get('aliases_to_change');

      // Mensaje indicando cuántos alias serán modificados.
      if ($aliases_to_change) {
        $count_aliases = count($aliases_to_change);
        $form['count_message'] = [
          '#markup' => $this->t('<strong>@count alias(es) serán modificados:</strong>', ['@count' => $count_aliases]),
          '#prefix' => '<div class="count-message">',
          '#suffix' => '</div>',
        ];

        // Mostrar la lista de cambios.
        $list_items = [];
        foreach ($aliases_to_change as $alias_change) {
          $list_items[] = $this->t('<li><s>@old</s> <strong>@new</strong></li>', [
            '@old' => $alias_change['old'],
            '@new' => $alias_change['new'],
          ]);
        }
        $form['preview_list'] = [
          '#markup' => '<ul>' . implode('', $list_items) . '</ul>',
        ];
      }

      // Ahora agregar el botón de aplicar cambios.
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Confirmar y Aplicar Cambios'),
      ];
    } else {
      $form['submit'] = [
        '#type' => 'submit',
        '#value' => $this->t('Previsualizar Cambios'),
      ];
    }

    return $form;
  }

  /**
   * Form submission handler.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Obtener valores del formulario.
    $search_string = $form_state->getValue('search_string');
    $replace_string = $form_state->getValue('replace_string');

    // Buscar todos los alias que contienen la cadena de búsqueda.
    $query = \Drupal::entityQuery('path_alias')
      ->condition('alias', '%' . $search_string . '%', 'LIKE')
      ->accessCheck(FALSE);
    $results = $query->execute();

    if ($form_state->get('preview_step') == '1') {
      // Si estamos en el paso de vista previa, aplicar los cambios.
      $this->startBatchProcess($results, $search_string, $replace_string);
    } else {
      // Paso de vista previa.
      if (!empty($results)) {
        $aliases_to_change = $this->getAliasChanges($results, $search_string, $replace_string);

        // Establecer el paso de vista previa como verdadero y almacenar los alias a cambiar en form_state.
        $form_state->set('preview_step', '1');
        $form_state->set('aliases_to_change', $aliases_to_change);
        $form_state->setRebuild(TRUE);
      } else {
        \Drupal::messenger()->addMessage($this->t('No se encontraron alias para la cadena de búsqueda dada.'), 'warning');
      }
    }
  }

  /**
   * Inicia el proceso de batch.
   */
  protected function startBatchProcess($results, $search_string, $replace_string) {
    $batch = [
      'title' => $this->t('Reemplazando aliases...'),
      'operations' => [],
      'finished' => [$this, 'batchFinished'],
    ];

    // Dividir el trabajo en operaciones.
    foreach ($results as $alias_id) {
      $batch['operations'][] = [
        [$this, 'replaceAlias'],
        [$alias_id, $search_string, $replace_string],
      ];
    }

    // Iniciar el batch.
    batch_set($batch);
  }

  /**
   * Operación de reemplazo de alias.
   */
  public function replaceAlias($alias_id, $search_string, $replace_string, array &$context) {
    // Cargar el alias de la base de datos.
    $path_alias = PathAlias::load($alias_id);
    if ($path_alias) {
      $old_alias = $path_alias->getAlias();
      $new_alias = str_replace($search_string, $replace_string, $old_alias);
      $path_alias->setAlias($new_alias);
      $path_alias->save();
    }

    // Actualizar el contexto del batch.
    $context['results']['replaced'][] = $alias_id;
  }

  /**
   * Función que se llama al finalizar el batch.
   */
  public function batchFinished($success, array $results, array $operations) {
    if ($success) {
      $count = count($results['replaced']);
      \Drupal::messenger()->addMessage($this->t('@count alias(es) han sido reemplazados exitosamente.', ['@count' => $count]));
    } else {
      \Drupal::messenger()->addMessage($this->t('Hubo un problema al reemplazar los alias.'), 'error');
    }
  }

  /**
   * Helper function to retrieve alias changes for preview.
   */
  protected function getAliasChanges($results, $search_string, $replace_string) {
    $aliases_to_change = [];
    foreach ($results as $alias_id) {
      $path_alias = PathAlias::load($alias_id);
      if ($path_alias) {
        $old_alias = $path_alias->getAlias();
        $new_alias = str_replace($search_string, $replace_string, $old_alias);
        $aliases_to_change[] = [
          'old' => $old_alias,
          'new' => $new_alias,
        ];
      }
    }
    return $aliases_to_change;
  }
}